% Test of the decoder: operation
clear all;

% Simulation parms
Fs = 64/7; % MHz
cndb = 12; % dB
k_opt = ' -2k '; % FFT size
kr_opt = ' -m 2k '; % FFT size, new receiver
M_opt = ' -M 4 '; % mapping
a_opt = ' -a NH '; % uniformity
D_opt = ' -D 1/32 '; % cyclic prefix
p_opt = ' -p 1/2 '; % puncturing rate
I = 1; % upsampling rate

% Analysis parms
fftsize = 1024;
window = blackman(fftsize);
noverlap = 128;
randn('seed', 23525);
timepeekwin = [1:40000];

Ts = 1 / Fs;

% Transmission channels
h = [1];
%h = [1 0.1 -0.2j];
%h = [1 0 0 -0.2+0.1j 0.1 0 0 0 0 0.4j];
%load P1; tau = floor(1e6 * t / Ts) + 1; h = zeros(1, max(tau)); for aa = 1:length(tau), h(tau(aa)) = h(tau(aa)) + h_t(aa); end;
%load F1; tau = floor(1e6 * t / Ts) + 1; h = zeros(1, max(tau)); for aa = 1:length(tau), h(tau(aa)) = h(tau(aa)) + h_t(aa); end;

% Normalize energy
h = h ./ sqrt(sum(h .* conj(h)));

% Run the transmission side
dos(['dvbtenco -i test.m2v -o test.bin -d ercvpbsmft -v 0 -c 0 ' k_opt M_opt a_opt D_opt p_opt ' -I ' int2str(I) 'x']);

% Load the enco tx time file
fid = fopen('test.bin', 'rb');
[tx_tx, txtx_len] = fread(fid, Inf, 'float32');
fclose(fid);
tx_tx = tx_tx(1:2:end) + j.*tx_tx(2:2:end);
txtx_len = txtx_len / 2;
disp(['Read ' num2str(txtx_len) ' TX TimeXDom.']);

% Spectrum of the transmitted
[Stx_tx, ftx_tx] = psd(tx_tx, fftsize, Fs * I, window, noverlap);
avg_spectxdb = mean(10*log10(Stx_tx(1:80)));

% Find the Noise Bandwidth
Bn = 1.0 * sum(Stx_tx) * (Fs/fftsize) * 10^(-avg_spectxdb/10);

% Pass through the channel
Ptx_tx = mean(abs(tx_tx).^2)
lentx = txtx_len;
h = h ./ sqrt(sum(abs(h).^2));
rx_tx = conv(tx_tx, h); 
rx_tx = rx_tx(1:lentx);

% Add noise
Prx_tx = mean(abs(rx_tx).^2)
sigma = sqrt(0.5 * Prx_tx * (Fs / Bn) * 10^(-cndb/10));
wgn = sigma .* (randn(size(rx_tx)) + j.*randn(size(rx_tx)));
Pwgn = mean(abs(wgn).^2)
rx_tx = rx_tx + wgn;
rxtx_len = length(rx_tx);
clear wgn;

% Write the channeled signal
fid = fopen('testcn.bin', 'wb');
fwrite(fid, [real(rx_tx) imag(rx_tx)].', 'float32');
fclose(fid);

% Run the receiver
dos(['dvbtdeco -i testcn.bin -o detest.m2v -d tfmqhsbpvcreg -v 1 -H lin -E zf -c 0 -s 100 ' kr_opt M_opt a_opt D_opt p_opt ' -I ' int2str(I) 'x']);

% Adjust lengths
tx_tx = tx_tx(1:min(txtx_len, rxtx_len));
rx_tx = rx_tx(1:min(txtx_len, rxtx_len));

% Spectrum of the arriving
[Srx_tx, frx_tx] = psd(rx_tx, fftsize, Fs * I, window, noverlap);
avg_specrxdb = mean(10*log10(Srx_tx(1:40)));
figure(1);
clf;
plot(frx_tx - I * Fs/2, fftshift(10*log10(Srx_tx)) - avg_specrxdb, 'b', ...
   ftx_tx - I * Fs/2, fftshift(10*log10(Stx_tx)) - avg_spectxdb, 'r');
grid;
legend('RX', 'TX', 0);
xlabel('Frequency (MHz)');
ylabel('PSD (dB_c)')
title('Power spectra');

%clear tx_tx rx_tx

% Load the reference, 1x enco td time file
fid = fopen('test1x.bin', 'rb');
[tx_td, txtd_len] = fread(fid, Inf, 'float32');
fclose(fid);
tx_td = tx_td(1:2:end) + j.*tx_td(2:2:end);
txtd_len = txtd_len / 2;
disp(['Read ' num2str(txtd_len) ' TX TimeDom.']);

% Run the decimator in matlab
if I > 1,
   load(['filter' int2str(I) 'x.mat']);
   mat_rx_td1 = filter(intfiltrzmin, 1, rx_tx);
   mat_rx_td1 = I * mat_rx_td1((15*I + (I - 1)):I:end); %15*4 + 3
   
   mat_rx_td2 = zeros(size(mat_rx_td1));
   %for mm = 1:4,
   %   mat_rx_td2 = mat_rx_td2 + 4 * filter(intfiltrzmin(mm:4:end), 1, rx_tx(mm:4:end));
   %end;
else,
   mat_rx_td1 = filter(1, 1, rx_tx);   
   mat_rx_td2 = zeros(size(mat_rx_td1));
end;

% Load the deco td time file
fid = fopen('detest_td.bin', 'rb');
[rx_td, rxtd_len] = fread(fid, Inf, 'float32');
fclose(fid);
rx_td = rx_td(1:2:end) + j.*rx_td(2:2:end);
rxtd_len = rxtd_len / 2;
disp(['Read ' num2str(rxtd_len) ' RX TimeDom.']);

% Adjust lengths
tx_td = tx_td(1:min(txtd_len, rxtd_len));
rx_td = rx_td(1:min(txtd_len, rxtd_len));

% Adjust for filtering delay
%rx_td = rx_td(8:end);
%tx_td = tx_td(1:(end - 7));

% Plot the time signal and compute the MER
merdb = 10*log10(sum(abs(tx_td)) ./ sum(abs(tx_td - rx_td).^2));
disp(['MER = ' num2str(merdb, '%.2f') ' dB']);
figure(2);
clf;
subplot(211);
%%timepeekwin = [1:length(rx_td)];
plot(timepeekwin - 1, real(rx_td(timepeekwin)), 'b', ...
   timepeekwin - 1, real(tx_td(timepeekwin)), 'r', ...
   timepeekwin - 1, real(mat_rx_td1(timepeekwin)), 'g', ...
   timepeekwin - 1, real(mat_rx_td2(timepeekwin)), 'm');
title(['Time plot - MER = ' num2str(merdb, '%.2f') ' dB']);
ylabel('Real part');
legend('RX', 'TX', 'MATRX1', 'MATRX2', 0);
subplot(212);
plot(timepeekwin - 1, imag(rx_td(timepeekwin)), 'b', ...
   timepeekwin - 1, imag(tx_td(timepeekwin)), 'r', ...
   timepeekwin - 1, imag(mat_rx_td1(timepeekwin)), 'g', ...
   timepeekwin - 1, imag(mat_rx_td2(timepeekwin)), 'm');
xlabel('Sample');
ylabel('Imaginary part');
legend('RX', 'TX', 'MATRX1', 'MATRX2', 0);

% Spectrum of the arriving
[Srx_td, frx_td] = psd(rx_td, fftsize, Fs, window, noverlap);
avg_specrxdb = mean(10*log10(Srx_td(1:40)));
figure(15);
clf;
plot(frx_td - Fs/2, fftshift(10*log10(Srx_td)) - avg_specrxdb, 'b');
grid;
legend('RX', 'TX', 0);
xlabel('Frequency (MHz)');
ylabel('PSD (dB_c)')
title('Power spectra');

clear tx_td rx_td

% Load the enco fd frequency file
fid = fopen('test_fd.bin', 'rb');
[tx_fd, txfd_len] = fread(fid, Inf, 'float32');
fclose(fid);
tx_fd = tx_fd(1:2:end) + j.*tx_fd(2:2:end);
txfd_len = txfd_len / 2;
disp(['Read ' num2str(txfd_len) ' TX FreqDom.']);

% Load the deco fd frequency file
fid = fopen('detest_fd.bin', 'rb');
[rx_fd, rxfd_len] = fread(fid, Inf, 'float32');
fclose(fid);
rx_fd = rx_fd(1:2:end) + j.*rx_fd(2:2:end);
rxfd_len = rxfd_len / 2;
disp(['Read ' num2str(rxfd_len) ' RX FreqDom.']);

% Adjust lengths
tx_fd = tx_fd(1:min(txfd_len, rxfd_len));
rx_fd = rx_fd(1:min(txfd_len, rxfd_len));

% Plot the frequency scatterplot and compute the MER
merdb = 10*log10(sum(abs(tx_fd)) ./ sum(abs(tx_fd - rx_fd).^2));
disp(['MER = ' num2str(merdb, '%.2f') ' dB']);
figure(3);
clf;
plot(real(tx_fd(timepeekwin)), imag(tx_fd(timepeekwin)), 'b.', ...
   real(rx_fd(timepeekwin)), imag(rx_fd(timepeekwin)), 'r.');
title(['Scatter plot of all carriers - MER = ' num2str(merdb, '%.2f') ' dB']);
xlabel('Real part');
ylabel('Imaginary part');
legend('TX', 'RX', 0);

clear tx_fd rx_fd

% Load the enco ma mapping file
fid = fopen('test_ma.bin', 'rb');
[tx_ma, txma_len] = fread(fid, Inf, 'float32');
fclose(fid);
tx_ma = tx_ma(1:2:end) + j.*tx_ma(2:2:end);
txma_len = txma_len / 2;
disp(['Read ' num2str(txma_len) ' TX Mapped']);

% Load the deco ma mapping file
fid = fopen('detest_ma.bin', 'rb');
[rx_ma, rxma_len] = fread(fid, Inf, 'float32');
fclose(fid);
rx_ma = rx_ma(1:2:end) + j.*rx_ma(2:2:end);
rxma_len = rxma_len / 2;
disp(['Read ' num2str(rxma_len) ' RX Mapped']);

% Load the deco eq equalizing file
fid = fopen('detest_eq.bin', 'rb');
[rx_eq, rxeq_len] = fread(fid, Inf, 'float32');
fclose(fid);
rx_eq = rx_eq(1:2:end) + j.*rx_eq(2:2:end);
rxeq_len = rxeq_len / 2;
disp(['Read ' num2str(rxeq_len) ' RX Equalized']);

% Adjust lengths
tx_ma = tx_ma(1:min([txma_len rxma_len rxeq_len]));
rx_ma = rx_ma(1:min([txma_len rxma_len rxeq_len]));
rx_eq = rx_eq(1:min([txma_len rxma_len rxeq_len]));

% Plot the map scatterplot and compute the S/N
snr_madb = 10*log10(sum(abs(tx_ma)) ./ sum(abs(tx_ma - rx_ma).^2))
snr_eqdb = 10*log10(sum(abs(tx_ma)) ./ sum(abs(tx_ma - rx_eq).^2))
disp(['S/N map = ' num2str(snr_eqdb, '%.2f') ' dB']);
figure(4);
clf;
plot(real(tx_ma(timepeekwin)), imag(tx_ma(timepeekwin)), 'b.', ...
   real(rx_ma(timepeekwin)), imag(rx_ma(timepeekwin)), 'r.', ...
   real(rx_eq(timepeekwin)), imag(rx_eq(timepeekwin)), 'g.');
title(['Scatter plot of data carriers - S/N_{uneq} = ' num2str(snr_madb, '%.2f') ...
      ' dB, S/N_{eq} = ' num2str(snr_eqdb, '%.2f') ' dB']);
xlabel('Real part');
ylabel('Imaginary part');
legend('TX', 'RX', 'EQ', 0);
figure(5);
subplot(211);
plot(timepeekwin - 1, real(tx_ma(timepeekwin)), 'b', ...
   timepeekwin - 1, real(rx_eq(timepeekwin)), 'g');
ylabel('Real part');
legend('TX', 'EQ', 0);
subplot(212);
plot(timepeekwin - 1, imag(tx_ma(timepeekwin)), 'b', ...
   timepeekwin - 1, imag(rx_eq(timepeekwin)), 'g');
ylabel('Imaginary part');
legend('TX', 'EQ', 0);

% Load the deco ch channel file
fid = fopen('detest_ch.bin', 'rb');
[rx_ch, rxch_len] = fread(fid, Inf, 'float32');
fclose(fid);
rx_ch = rx_ch(1:2:end) + j.*rx_ch(2:2:end);
rxch_len = rxch_len / 2;
disp(['Read ' num2str(rxch_len) ' RX Channel']);

% Plot the estimated channel
figure(6);
clf;
subplot(211);
plot((timepeekwin - 1), 20*log10(abs(rx_ch(timepeekwin))));
ylabel('dB_c');
xlabel('Sample');
title('Estimated Channel H(f)');
legend('ABS', 0);
subplot(212);
plot((timepeekwin - 1), real(rx_ch(timepeekwin)), 'g', ...
   (timepeekwin - 1), imag(rx_ch(timepeekwin)), 'm');
xlabel('Sample');
ylabel('Amplitude');
legend('Re', 'Im', 0);

clear tx_ma rx_ma rx_eq rx_ch

% Load the enco si symbolfile
fid = fopen('test_si.bin', 'rb');
[tx_si, txsi_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(txsi_len) ' TX Symbol Interleaved']);

% Load the deco si symbolfile
fid = fopen('detest_si.bin', 'rb');
[rx_si, rxsi_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(rxsi_len) ' RX Symbol Interleaved']);

% Adjust lengths
tx_si = tx_si(1:min(txsi_len, rxsi_len));
rx_si = rx_si(1:min(txsi_len, rxsi_len));

% Find the symbol errors
[symerr, Perrs] = symerr(tx_si, rx_si)
figure(7);
subplot(211);
plot(tx_si(timepeekwin) ~= rx_si(timepeekwin));
title(['Errors = ' int2str(symerr) '; SER = ' num2str(Perrs, '%.1E')]);
ylabel('Symbol Int. errors');

clear tx_si rx_si;

% Load the enco bi bitfile
fid = fopen('test_bi.bin', 'rb');
[tx_bi, txbi_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(txbi_len) ' TX Bit Interleaved']);

% Load the deco bi bitfile
fid = fopen('detest_bi.bin', 'rb');
[rx_bi, rxbi_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(rxbi_len) ' RX Bit Interleaved']);

% Adjust lengths
tx_bi = tx_bi(1:min(txbi_len, rxbi_len));
rx_bi = rx_bi(1:min(txbi_len, rxbi_len));

% Find the symbol errors
[symerr, Perrs] = symerr(tx_bi, rx_bi)
figure(7);
subplot(212);
plot(tx_bi(timepeekwin) ~= rx_bi(timepeekwin));
title(['Errors = ' int2str(symerr) '; SER = ' num2str(Perrs, '%.1E')]);
ylabel('Bit Int. errors');

clear tx_bi rx_bi;

% Load the enco pu punfile
fid = fopen('test_pu.bin', 'rb');
[tx_pu, txpu_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(txpu_len) ' TX Punctured']);

% Load the deco pu punfile
fid = fopen('detest_pu.bin', 'rb');
[rx_pu, rxpu_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(rxpu_len) ' RX Punctured']);

% Adjust lengths
tx_pu = tx_pu(1:min(txpu_len, rxpu_len));
rx_pu = rx_pu(1:min(txpu_len, rxpu_len));

% Find the bit errors
[biterr, Perrb] = biterr(tx_pu, rx_pu)
figure(8);
subplot(211);
plot(tx_pu(timepeekwin) ~= rx_pu(timepeekwin));
title(['Errors = ' int2str(biterr) '; BER = ' num2str(Perrb, '%.1E')]);
ylabel('Punctured errors');

clear tx_pu rx_pu;

% Load the enco ve convfile
fid = fopen('test_ve.bin', 'rb');
[tx_ve, txve_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(txve_len) ' TX Encoded']);

% Load the deco ve convfile
fid = fopen('detest_ve.bin', 'rb');
[rx_ve, rxve_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(rxve_len) ' RX Encoded']);

% Adjust lengths
tx_ve = tx_ve(1:min(txve_len, rxve_len));
rx_ve = rx_ve(1:min(txve_len, rxve_len));

% Find the bit errors
[biterr, Perrb] = biterr(tx_ve, rx_ve)
figure(8);
subplot(212);
plot(tx_ve(timepeekwin) ~= rx_ve(timepeekwin));
title(['Errors = ' int2str(biterr) '; BER = ' num2str(Perrb, '%.1E')]);
ylabel('Encoded errors');

clear tx_ve rx_ve;

% Load the enco ci intfile
fid = fopen('test_ci.bin', 'rb');
[tx_ci, txci_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(txci_len) ' TX Interleaved']);

% Load the deco ci punfile
fid = fopen('detest_ci.bin', 'rb');
[rx_ci, rxci_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(rxci_len) ' RX Interleaved']);

% Adjust lengths
tx_ci = tx_ci(1:min(txci_len, rxci_len));
rx_ci = rx_ci(1:min(txci_len, rxci_len));

% Find the bit errors
[symerr, Perrs] = symerr(tx_ci, rx_ci)
figure(9);
subplot(211);
plot(tx_ci(timepeekwin) ~= rx_ci(timepeekwin));
title(['Errors = ' int2str(symerr) '; SER = ' num2str(Perrs, '%.1E')]);
ylabel('Interleaved errors');

clear tx_ci rx_ci;

% Load the enco rs rsfile
fid = fopen('test_rs.bin', 'rb');
[tx_rs, txrs_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(txrs_len) ' TX Reed-Solomoned']);

% Load the deco rs rsfile
fid = fopen('detest_rs.bin', 'rb');
[rx_rs, rxrs_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(rxrs_len) ' RX Reed-Solomoned']);

% Adjust lengths
tx_rs = tx_rs(1:min(txrs_len, rxrs_len));
rx_rs = rx_rs(1:min(txrs_len, rxrs_len));

% Adjust for deinterleaving delay
%rx_rs = rx_rs(2245:end);
%tx_rs = tx_rs(1:(end - 2244));

% Find the bit errors
[biterr, Perrb] = biterr(tx_rs, rx_rs)
figure(9);
subplot(212);
plot(tx_rs(timepeekwin) ~= rx_rs(timepeekwin));
title(['Errors = ' int2str(biterr) '; BER = ' num2str(Perrb, '%.1E')]);
ylabel('Reed-Solomoned errors');

clear tx_rs rx_rs;

% Load the enco ed energyfile
fid = fopen('test_ed.bin', 'rb');
[tx_ed, txed_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(txed_len) ' TX Energized']);

% Load the deco ed energyfile
fid = fopen('detest_ed.bin', 'rb');
[rx_ed, rxed_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(rxed_len) ' RX Energized']);

% Adjust lengths
tx_ed = tx_ed(1:min(txed_len, rxed_len));
rx_ed = rx_ed(1:min(txed_len, rxed_len));

% Find the bit errors
[biterr, Perrb] = biterr(tx_ed, rx_ed)
figure(10);
subplot(211);
plot(tx_ed(timepeekwin) ~= rx_ed(timepeekwin));
title(['Errors = ' int2str(biterr) '; BER = ' num2str(Perrb, '%.1E')]);
ylabel('Energized errors');

clear tx_ed rx_ed;

% Load the enco mp mpeg2file
fid = fopen('test.m2v', 'rb');
[tx_mp, txmp_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(txmp_len) ' TX MPEG-2']);

% Load the deco mp mpeg2file
fid = fopen('detest.m2v', 'rb');
[rx_mp, rxmp_len] = fread(fid, Inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(rxmp_len) ' RX MPEG-2']);

% Adjust lengths
tx_mp = tx_mp(1:min(txmp_len, rxmp_len));
rx_mp = rx_mp(1:min(txmp_len, rxmp_len));

% Find the bit errors
[biterr, Perrb] = biterr(tx_mp, rx_mp)
figure(10);
subplot(212);
plot(tx_mp(timepeekwin) ~= rx_mp(timepeekwin));
title(['Errors = ' int2str(biterr) '; BER = ' num2str(Perrb, '%.1E')]);
ylabel('MPEG-2 errors');

break;
clear tx_mp rx_mp;


